@@ -42,7 +42,7 @@ def handle_shutdown():
 
 
 class Popen(SPOpen):
-    """Inject default into Popen."""
+    """Inject defaults into Popen."""
 
     def __init__(self, *args, stderr=None, stdin=None, **kwargs):
         """Inject default into Popen."""
@@ -160,12 +160,11 @@ def git_try_fast_forward(rev, last):
     """Clone git-repo."""
     try:
         run(["git", "merge", "--no-commit", "--ff-only", rev], check=True)
+        wipe()
+        checkout(last)
         return True
     except CalledProcessError:
         return False
-    finally:
-        wipe()
-        checkout(last)
 
 
 def git_clone(source, destination):
@@ -203,7 +202,15 @@ def onelines(rev, *, last=None):
     """Get the short-message of a commit from git."""
     if last:
         res = run(
-            ["git", "log", "--oneline", "--no-decorate", f"{last}..{rev}"],
+            [
+                "git",
+                "log",
+                "--oneline",
+                "--no-decorate",
+                "--date-order",
+                "--no-merges",
+                f"{last}..{rev}",
+            ],
             stdout=PIPE,
             check=True,
         )
@@ -280,6 +287,7 @@ def get_rev_list(head, base):
             "--reverse",
             "--topo-order",
             "--ancestry-path",
+            "--no-merges",
             f"{base}..{head}",
         ],
         stdout=PIPE,
@@ -398,7 +406,7 @@ def checkpoint(rev):
 
 def warning():
     """Print a warning that git-darcs is going to wipe uncommitted change."""
-    print("Use git-darcs on an extra tracking repository.")
+    print("Use git-darcs on an extra tracking-repository.")
     print("git-darcs WILL CLEAR ALL YOUR WORK THAT IS NOT COMMITED!")
     print("Use -nw to skip this warning.\n")
     print("Press enter to continue")
@@ -525,7 +533,7 @@ def clone(source, destination, verbose):
     "-w/-nw",
     "--warn/--no-warn",
     default=True,
-    help="Warn that repository will be cleaned",
+    help="Warn that repository will be cleared",
 )
 @click.option(
     "--base",
@@ -573,6 +581,10 @@ def update(verbose, base, warn, shallow):
     branch = get_current_branch()
     failed = True
     try:
+        try:
+            checkout(".")
+        except CalledProcessError:
+            pass
         if do_one:
             import_one()
         else:
